@@ -9,6 +9,7 @@ from collections import OrderedDict
 from Qt import QtWidgets
 from Qt import QtGui
 from Qt import QtCore
+from PySide2 import __version_info__ as qt_version
 
 # Internal
 import nxt_editor
@@ -17,12 +18,23 @@ from nxt.nxt_layer import LAYERS
 from . import colors
 from nxt.stage import INTERNAL_ATTRS
 from .label_edit import NameEditDialog
+from .user_dir import USER_PREF, user_prefs
 
 
 logger = logging.getLogger(nxt_editor.LOGGER_NAME)
 
+MIN_LOD = user_prefs.get(USER_PREF.LOD, .4)
 
-class NodeGraphicsItem(QtWidgets.QGraphicsItem):
+_pyside_version = qt_version
+
+
+if _pyside_version[1] < 11:
+    graphic_type = QtWidgets.QGraphicsItem
+else:
+    graphic_type = QtWidgets.QGraphicsObject
+
+
+class NodeGraphicsItem(graphic_type):
     """The graphics item used to represent nodes in the graph. Contains
     instances of NodeGraphicsPlug for each attribute on the associated node.
     Contains functionality for arranging children into stacks.
@@ -44,11 +56,12 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
 
         # item settings
         self.setCacheMode(QtWidgets.QGraphicsItem.DeviceCoordinateCache)
-        #self.setCacheMode(QtWidgets.QGraphicsItem.ItemCoordinateCache)
+
         self.setFlags(QtWidgets.QGraphicsItem.ItemIsMovable |
                       QtWidgets.QGraphicsItem.ItemIsFocusable |
                       QtWidgets.QGraphicsItem.ItemIsSelectable |
-                      QtWidgets.QGraphicsItem.ItemSendsScenePositionChanges)
+                      QtWidgets.QGraphicsItem.ItemSendsScenePositionChanges |
+                      QtWidgets.QGraphicsItem.ItemNegativeZStacksBehindParent)
         self.setAcceptHoverEvents(True)
 
         # draw settings
@@ -85,6 +98,99 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
         # draw node
         self.update_from_model()
 
+        # Setup groups
+        # In
+        self.in_anim_group = QtCore.QParallelAnimationGroup()
+        self.in_anim_group.finished.connect(self.finished_anim)
+        # Out
+        self.out_anim_group = QtCore.QParallelAnimationGroup()
+        self.out_anim_group.finished.connect(self.finished_anim)
+
+    def _setup_anim_properties(self):
+        # Position anim property
+        self.pos_anim = QtCore.QPropertyAnimation(self, b"pos", self)
+        # Set graphics effect
+        effect = QtWidgets.QGraphicsOpacityEffect(self)
+        effect.setOpacity(1)
+        self.setGraphicsEffect(effect)
+        # Opacity anim property
+        self.opacity_anim = QtCore.QPropertyAnimation(effect, b"opacity",
+                                                      effect)
+        # Lower power caching
+        self.setCacheMode(QtWidgets.QGraphicsItem.ItemCoordinateCache)
+
+    def setup_in_anim(self):
+        self._setup_anim_properties()
+        self.in_anim_group.addAnimation(self.pos_anim)
+        self.in_anim_group.addAnimation(self.opacity_anim)
+
+    def setup_out_anim(self):
+        self._setup_anim_properties()
+        self.out_anim_group.addAnimation(self.pos_anim)
+        self.out_anim_group.addAnimation(self.opacity_anim)
+
+    def finished_anim(self):
+        self.setGraphicsEffect(None)
+        self.in_anim_group.clear()
+        self.out_anim_group.clear()
+        self.setCacheMode(QtWidgets.QGraphicsItem.DeviceCoordinateCache)
+        self.view.update()
+        self.view._animating.remove(self)
+
+    def get_is_animating(self):
+        i = self.in_anim_group.State.Running == self.in_anim_group.state()
+        o = self.out_anim_group.State.Running == self.out_anim_group.state()
+        return i or o
+
+    def anim_into_place(self, end_pos):
+        if self.get_is_animating():
+            return
+        if end_pos == self.pos():
+            return
+        self.view._animating.append(self)
+        if self.view.do_animations:
+            self.setup_in_anim()
+        else:
+            self.setPos(end_pos)
+            self.in_anim_group.finished.emit()
+            return
+
+        self.opacity_anim.setStartValue(0)
+        self.opacity_anim.setEndValue(1)
+        self.opacity_anim.setDuration(80)
+
+        curve = QtCore.QEasingCurve(QtCore.QEasingCurve.OutBack)
+        curve.setAmplitude(.8)
+        self.pos_anim.setEasingCurve(curve)
+
+        self.pos_anim.setDuration(100)
+        self.pos_anim.setEndValue(end_pos)
+        self.in_anim_group.start()
+
+    def anim_out(self):
+        if self.get_is_animating():
+            return
+        self.view._animating.append(self)
+        if self.view.do_animations:
+            self.setup_out_anim()
+        else:
+            self.out_anim_group.finished.emit()
+            return
+        self.setCacheMode(QtWidgets.QGraphicsItem.ItemCoordinateCache)
+        self.opacity_anim.setStartValue(1)
+        self.opacity_anim.setEndValue(0)
+        self.opacity_anim.setDuration(80)
+
+        self.pos_anim.setDuration(80)
+        self.pos_anim.setEasingCurve(QtCore.QEasingCurve.Linear)
+        x_move = self.stack_offset * -1 * .5
+        if not self.parentItem() or not self.parentItem().parentItem():
+            y_move = 0.0
+        else:
+            y_move = (self.parentItem().boundingRect().height() * -1.0) * .5
+        self.pos_anim.setEndValue(QtCore.QPointF(x_move, y_move))
+        self.out_anim_group.start()
+
     def update_color(self):
         layers = self.model.get_layers_with_opinion(self.node_path)
         n_colors = []
@@ -160,7 +266,7 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
             ml = QtWidgets.QApplication.mouseButtons() == QtCore.Qt.LeftButton
             shift = QtWidgets.QApplication.keyboardModifiers() == QtCore.Qt.ShiftModifier
             force_snap = self.view.alignment_actions.snap_action.isChecked()
-            if (ml & shift) or force_snap:
+            if (ml & shift) or force_snap and not self.get_is_animating():
                 value = self.closest_grid_point(value)
                 return value
 
@@ -176,16 +282,22 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
         """Override of QtWidgets.QGraphicsItem paint. Handles all visuals of the Node. Split up into 3
         functions for organization.
         """
-        painter.setRenderHints(QtGui.QPainter.Antialiasing | QtGui.QPainter.TextAntialiasing | QtGui.QPainter.SmoothPixmapTransform)
-        self.draw_title(painter)
-        self.draw_attributes(painter)
-        self.draw_border(painter)
+        lod = QtWidgets.QStyleOptionGraphicsItem.levelOfDetailFromTransform(painter.worldTransform())
+        if lod > MIN_LOD:
+            painter.setRenderHints(QtGui.QPainter.Antialiasing |
+                                   QtGui.QPainter.TextAntialiasing |
+                                   QtGui.QPainter.SmoothPixmapTransform)
+        else:
+            painter.setRenderHints(False)
+        self.draw_title(painter, lod)
+        self.draw_attributes(painter, lod)
+        self.draw_border(painter, lod)
 
     def closest_grid_point(self, position):
         snapped_pos = self.model.snap_pos_to_grid((position.x(), position.y()))
         return QtCore.QPointF(*snapped_pos)
 
-    def draw_border(self, painter):
+    def draw_border(self, painter, lod=1.):
         """Draws border, called exclusively by paint.
 
         :param painter: painter from paint.
@@ -213,7 +325,7 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
                          self.get_selection_rect().width() - 2,
                          self.get_selection_rect().height() - 2))
 
-    def draw_title(self, painter):
+    def draw_title(self, painter, lod=1.):
         """Draw title of the node. Called exclusively in paint.
 
         :param painter: painter from paint.
@@ -307,76 +419,92 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
             if self.exec_out_plug:
                 self.scene().removeItem(self.exec_out_plug)
                 self.exec_out_plug = None
-
-        # draw attr dots
-        offset = -6
-        for fill in self.attr_dots:
-            painter.setBrush(QtCore.Qt.white)
-            if fill:
+        if lod > MIN_LOD:
+            # draw attr dots
+            offset = -6
+            for fill in self.attr_dots:
                 painter.setBrush(QtCore.Qt.white)
-            else:
-                painter.setBrush(QtCore.Qt.NoBrush)
-            dots_color = QtGui.QColor(QtCore.Qt.white).darker(self.dim_factor)
-            painter.setPen(QtGui.QPen(dots_color, 0.5))
-            dot_x = self.max_width - 15
-            dot_y = (self.title_rect_height / 2) + offset
-            painter.drawEllipse(QtCore.QPointF(dot_x, dot_y), 2, 2)
-            offset += 6
+                if fill:
+                    painter.setBrush(QtCore.Qt.white)
+                else:
+                    painter.setBrush(QtCore.Qt.NoBrush)
+                dots_color = QtGui.QColor(QtCore.Qt.white).darker(self.dim_factor)
+                painter.setPen(QtGui.QPen(dots_color, 0.5))
+                dot_x = self.max_width - 15
+                dot_y = (self.title_rect_height / 2) + offset
+                painter.drawEllipse(QtCore.QPointF(dot_x, dot_y), 2, 2)
+                offset += 6
 
         # draw title
-        painter.setPen(QtGui.QColor(QtCore.Qt.white).darker(self.dim_factor))
-        if not self.node_enabled:
-            painter.setPen(QtGui.QColor(QtCore.Qt.white).darker(150))
+
         painter.setFont(self.title_font)
         title_str = nxt_path.node_name_from_node_path(self.node_path)
         font_metrics = QtGui.QFontMetrics(self.title_font)
         width = self.max_width - 40
         if self.error_list:
             width -= 20
-        title = font_metrics.elidedText(title_str, QtCore.Qt.ElideRight, width)
-        painter.drawText(15, 0, self.max_width - 15, self.title_rect_height,
-                         QtCore.Qt.AlignVCenter, title)
-
-        # draw error
-        if self.error_list:
-            pos = QtCore.QPointF(self.max_width-45, self.title_rect_height/4)
-            error_item = ErrorItem(font=QtGui.QFont('Roboto', 16, 75),
-                                   pos=pos, text='!')
-            error_item.setParentItem(self)
-            error_item.setZValue(50)
-            self.error_item = error_item
+        if lod > MIN_LOD:
+            painter.setPen(
+                QtGui.QColor(QtCore.Qt.white).darker(self.dim_factor))
+            if not self.node_enabled:
+                painter.setPen(QtGui.QColor(QtCore.Qt.white).darker(150))
+            title = font_metrics.elidedText(title_str,
+                                            QtCore.Qt.ElideRight, width)
+            painter.drawText(15, 0, self.max_width - 15, self.title_rect_height,
+                             QtCore.Qt.AlignVCenter, title)
         else:
-            if self.error_item:
-                self.scene().removeItem(self.error_item)
-                self.error_item.deleteLater()
-            self.error_item = None
+            painter.setBrush(QtGui.QColor(QtCore.Qt.white).darker(self.dim_factor))
+            if not self.node_enabled:
+                painter.setBrush(QtGui.QColor(QtCore.Qt.white).darker(150))
+            proxy_rect = font_metrics.boundingRect(title_str)
+            r_width = proxy_rect.width() * .8
+            height = proxy_rect.height()
+            painter.drawRect(15, height * .8,
+                             min(r_width, width), height * .2)
 
+        if lod > MIN_LOD:
+            # draw error
+            if self.error_list:
+                pos = QtCore.QPointF(self.max_width-45, self.title_rect_height/4)
+                error_item = ErrorItem(font=QtGui.QFont('Roboto', 16, 75),
+                                       pos=pos, text='!')
+                error_item.setParentItem(self)
+                error_item.setZValue(50)
+                self.error_item = error_item
+            else:
+                if self.error_item:
+                    self.scene().removeItem(self.error_item)
+                    self.error_item.deleteLater()
+                self.error_item = None
+
+        # draw collapse state arrow
         for arrow in self.collapse_arrows:
             self.scene().removeItem(arrow)
-        self.collapse_arrows = []
-        # TODO calculation needed arrows should be done outside drawing
-        # draw collapse state arrow
-        if self.collapse_state:
-            des_colors = self.model.get_descendant_colors(self.node_path)
-            filled = self.model.has_children(self.node_path)
-            if not filled:
-                des_colors = [QtCore.Qt.white]
-            elif not des_colors:
-                disp = self.model.comp_layer
-                des_colors = [self.model.get_node_color(self.node_path, disp)]
-            i = 0
-            num = len(des_colors)
-            for c in des_colors:
-                arrow = CollapseArrow(self, filled=filled, color=c)
-                arrow_width = arrow.width * 1.1
-                center_offset = (arrow_width * (num * .5) - arrow_width * .5)
-                cur_offset = (i * arrow_width)
-                pos = ((self.max_width * .5) + center_offset - cur_offset)
-                arrow.setPos(pos, self.title_rect_height)
-                self.collapse_arrows += [arrow]
-                i += 1
+        if lod > MIN_LOD:
+            self.collapse_arrows = []
+            # TODO calculation needed arrows should be done outside drawing
 
-    def draw_attributes(self, painter):
+            if self.collapse_state:
+                des_colors = self.model.get_descendant_colors(self.node_path)
+                filled = self.model.has_children(self.node_path)
+                if not filled:
+                    des_colors = [QtCore.Qt.white]
+                elif not des_colors:
+                    disp = self.model.comp_layer
+                    des_colors = [self.model.get_node_color(self.node_path, disp)]
+                i = 0
+                num = len(des_colors)
+                for c in des_colors:
+                    arrow = CollapseArrow(self, filled=filled, color=c)
+                    arrow_width = arrow.width * 1.1
+                    center_offset = (arrow_width * (num * .5) - arrow_width * .5)
+                    cur_offset = (i * arrow_width)
+                    pos = ((self.max_width * .5) + center_offset - cur_offset)
+                    arrow.setPos(pos, self.boundingRect().height())
+                    self.collapse_arrows += [arrow]
+                    i += 1
+
+    def draw_attributes(self, painter, lod=1.):
         """Draw attributes for this node. Called exclusively by paint.
 
         :param painter: painter from paint.
@@ -395,44 +523,60 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
             self._attr_plug_graphics.setdefault(attr_name, {})
             attr_plug_graphics = self._attr_plug_graphics[attr_name]
             current_in_plug = attr_plug_graphics.get('in_plug')
-            in_pos = self.get_attr_in_pos(attr_name, scene=False)
-            if current_in_plug:
-                current_in_plug.setPos(in_pos)
-                current_in_plug.color = target_color
-                current_in_plug.update()
-            else:
-                in_plug = NodeGraphicsPlug(pos=in_pos,
-                                           radius=self.ATTR_PLUG_RADIUS,
-                                           color=target_color,
-                                           attr_name_represented=attr_name,
-                                           is_input=True)
-                attr_plug_graphics['in_plug'] = in_plug
-                in_plug.setParentItem(self)
+            if lod > MIN_LOD:
+                in_pos = self.get_attr_in_pos(attr_name, scene=False)
+                if current_in_plug:
+                    current_in_plug.show()
+                    current_in_plug.setPos(in_pos)
+                    current_in_plug.color = target_color
+                    current_in_plug.update()
+                else:
+                    current_in_plug = NodeGraphicsPlug(pos=in_pos,
+                                                       radius=self.ATTR_PLUG_RADIUS,
+                                                       color=target_color,
+                                                       attr_name_represented=attr_name,
+                                                       is_input=True)
+                    attr_plug_graphics['in_plug'] = current_in_plug
+                    current_in_plug.setParentItem(self)
+            elif current_in_plug:
+                current_in_plug.hide()
 
             current_out_plug = attr_plug_graphics.get('out_plug')
-            out_pos = self.get_attr_out_pos(attr_name, scene=False)
-            if current_out_plug:
-                current_out_plug.setPos(out_pos)
-                current_out_plug.color = target_color
-                current_out_plug.update()
-            else:
-                out_plug = NodeGraphicsPlug(pos=out_pos,
-                                            radius=self.ATTR_PLUG_RADIUS,
-                                            color=target_color,
-                                            attr_name_represented=attr_name,
-                                            is_input=False)
-                attr_plug_graphics['out_plug'] = out_plug
-                out_plug.setParentItem(self)
+            if lod > MIN_LOD:
+                out_pos = self.get_attr_out_pos(attr_name, scene=False)
+                if current_out_plug:
+                    current_out_plug.show()
+                    current_out_plug.setPos(out_pos)
+                    current_out_plug.color = target_color
+                    current_out_plug.update()
+                else:
+                    out_plug = NodeGraphicsPlug(pos=out_pos,
+                                                radius=self.ATTR_PLUG_RADIUS,
+                                                color=target_color,
+                                                attr_name_represented=attr_name,
+                                                is_input=False)
+                    attr_plug_graphics['out_plug'] = out_plug
+                    out_plug.setParentItem(self)
+            elif current_out_plug:
+                current_out_plug.hide()
 
             # draw attr_name
-            painter.setPen(attr_details['title_color'])
-            painter.setFont(attr_details['title_font'])
             rect = attr_details['bg_rect']
+            painter.setFont(attr_details['title_font'])
             font_metrics = QtGui.QFontMetrics(self.attr_font)
             title = font_metrics.elidedText(attr_name, QtCore.Qt.ElideRight,
                                             self.max_width - 20)
-            painter.drawText(rect.x() + 10, rect.y() - 1, rect.width(),
-                             rect.height(), QtCore.Qt.AlignVCenter, title)
+            if lod > MIN_LOD:
+                painter.setPen(attr_details['title_color'])
+                painter.drawText(rect.x() + 10, rect.y() - 1, rect.width(),
+                                 rect.height(), QtCore.Qt.AlignVCenter, title)
+            else:
+                proxy_rect = font_metrics.boundingRect(title)
+                height = proxy_rect.height()
+                width = proxy_rect.width()
+                painter.setBrush(attr_details['title_color'].darker(150))
+                painter.drawRect(rect.x() + 10, rect.y() + height*.8,
+                                 width, height*.2)
 
     def calculate_attribute_draw_details(self):
         """Calculate position of all known attr names. Details stored in
@@ -721,9 +865,8 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
         children_paths = self.model.get_children(self.node_path, ordered=True,
                                                  include_implied=True)
         prev_y = 0
-        offset_z = len(children_paths)
         prev_child = None
-        index = 0
+        index = 1
         for child_path in children_paths:
             child = self.view.get_node_graphic(child_path)
             if not child:
@@ -737,11 +880,9 @@ class NodeGraphicsItem(QtWidgets.QGraphicsItem):
             else:
                 y = self.get_selection_rect().height()
             y += prev_y
-            child.setPos(self.stack_offset, y)
-            child.setZValue(offset_z)
-
+            new_pos = QtCore.QPointF(self.stack_offset, y)
+            child.anim_into_place(new_pos)
             prev_y = y
-            offset_z -= 1
             child.arrange_descendants()
             prev_child = child
             index += 1
@@ -824,7 +965,14 @@ class NodeGraphicsPlug(QtWidgets.QGraphicsItem):
 
     def paint(self, painter, option, widget):
         """Override of QtWidgets.QGraphicsItem paint. Handles all visuals of the Plug."""
-        painter.setRenderHints(QtGui.QPainter.Antialiasing | QtGui.QPainter.TextAntialiasing | QtGui.QPainter.SmoothPixmapTransform)
+        lod = QtWidgets.QStyleOptionGraphicsItem.levelOfDetailFromTransform(
+            painter.worldTransform())
+        if lod > MIN_LOD:
+            painter.setRenderHints(QtGui.QPainter.Antialiasing |
+                                   QtGui.QPainter.TextAntialiasing |
+                                   QtGui.QPainter.SmoothPixmapTransform)
+        else:
+            painter.setRenderHints(False)
         if self.is_hovered:
             painter.setPen(QtGui.QPen(QtCore.Qt.white, self.hover_width, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
         else:
@@ -836,7 +984,7 @@ class NodeGraphicsPlug(QtWidgets.QGraphicsItem):
             # create triangle
             polygon = QtGui.QPolygonF()
             step_angle = 120
-            for i in range(4):
+            for i in [0, 1, 2, 3]:
                 step = step_angle * i
                 x = self.radius * 1.2 * math.cos(math.radians(step))
                 y = self.radius * 1.2 * math.sin(math.radians(step))
@@ -853,8 +1001,6 @@ class NodeGraphicsPlug(QtWidgets.QGraphicsItem):
             painter.setBrush(QtCore.Qt.green)
             painter.drawPolygon(polygon)
 
-            painter.setPen(QtCore.Qt.black)
-            painter.setFont(QtGui.QFont('Roboto', 12))
         elif self.is_break:
             painter.drawRect(self.radius * -1, self.radius * -1, self.radius * 2, self.radius * 2)
         else:
@@ -931,6 +1077,7 @@ class CollapseArrow(QtWidgets.QGraphicsItem):
             is_str = isinstance(self.color, str)
         if is_str:
             self.color = QtGui.QColor(self.color)
+        self.setZValue(30)
 
     def boundingRect(self):
         """Override of QtWidgets.QGraphicsItem boundingRect. If this rectangle
@@ -946,18 +1093,35 @@ class CollapseArrow(QtWidgets.QGraphicsItem):
         if self.filled:
             brush = QtGui.QBrush(self.color)
             painter.setBrush(brush)
+            if self.color.lightness() < 100:
+                pen_color = QtGui.QColor(self.color.lighter(300))
+            else:
+                pen_color = QtGui.QColor(self.color.darker(300))
+            pen = QtGui.QPen(pen_color)
+            pen.setJoinStyle(QtCore.Qt.RoundJoin)
             painter.setPen(QtCore.Qt.NoPen)
         else:
-            painter.setBrush(QtCore.Qt.NoBrush)
-            painter.setPen(QtCore.Qt.white)
+            brush = QtGui.QBrush(QtCore.Qt.white, QtCore.Qt.Dense6Pattern)
+            painter.setBrush(brush)
+            pen = QtGui.QPen(QtCore.Qt.white)
+            pen.setStyle(QtCore.Qt.DotLine)
+            painter.setPen(pen)
 
         # draw triangle
         points = [
-            QtCore.QPointF(0-(self.width*.5), 0),
+            QtCore.QPointF(0 - (self.width * .5), 0),
             QtCore.QPointF(self.width*.5, 0),
             QtCore.QPointF(0, self.height)
         ]
         painter.drawPolygon(points)
+        # Draw outline
+        if self.filled:
+            painter.setBrush(QtCore.Qt.NoBrush)
+            painter.setPen(pen)
+            painter.drawLine(0 - (self.width * .5), 0,
+                             0, self.height)
+            painter.drawLine(0, self.height,
+                             self.width * .5, 0)
 
     def itemChange(self, change, value):
         """Override of QtWidgets.QGraphicsItem itemChange."""
